@@ -1,4 +1,4 @@
-#  Copyright © 2020 Ingram Micro Inc. All rights reserved.
+#  Copyright © 2021 Ingram Micro Inc. All rights reserved.
 
 ALL_BASIC_FIELDS = '__all__'
 
@@ -20,3 +20,9 @@ class SignalType:
 
 
 NO_QUEUE = 'None'
+
+DEFAULT_DEAD_MESSAGE_TTL = 864000  # 10 days
+DEFAULT_DELAY_QUEUE_MAX_SIZE = None  # Infinite
+DEFAULT_CQRS_MESSAGE_TTL = 86400  # 1 day
+DEFAULT_CQRS_MAX_RETRIES = 30
+DEFAULT_CQRS_RETRY_DELAY = 2  # seconds
