@@ -1,5 +1,7 @@
 #  Copyright © 2021 Ingram Micro Inc. All rights reserved.
 
+import logging
+
 from django.conf import settings
 from django.db import router, transaction
 from django.db.models import DateField, DateTimeField, F, IntegerField, Manager, Model
@@ -8,6 +10,8 @@ from django.utils.module_loading import import_string
 
 from dj_cqrs.constants import (
     ALL_BASIC_FIELDS,
+    DEFAULT_CQRS_MAX_RETRIES,
+    DEFAULT_CQRS_RETRY_DELAY,
     FIELDS_TRACKER_FIELD_NAME,
     TRACKED_FIELDS_ATTR_NAME,
 )
@@ -16,6 +20,9 @@ from dj_cqrs.metas import MasterMeta, ReplicaMeta
 from dj_cqrs.signals import MasterSignals, post_bulk_create, post_update
 
 
+logger = logging.getLogger('django-cqrs')
+
+
 class RawMasterMixin(Model):
 
     """Base class for MasterMixin. **Users shouldn't use this
@@ -404,3 +411,55 @@ class ReplicaMixin(Model, metaclass=ReplicaMeta):
             raise NotImplementedError
 
         return cls.cqrs.delete_instance(master_data)
+
+    @staticmethod
+    def should_retry_cqrs(current_retry, exception=None):
+        """Checks if we should retry the message after current attempt.
+
+        :param current_retry: Current number of message retries.
+        :type current_retry: int
+        :param exception: Exception instance raised during message consume.
+        :type exception: Exception, optional
+        :return: True if message should be retried, False otherwise.
+        :rtype: bool
+        """
+        replica_settings = settings.CQRS.get('replica', {})
+        if 'CQRS_MAX_RETRIES' in replica_settings and replica_settings['CQRS_MAX_RETRIES'] is None:
+            # Infinite
+            return True
+
+        min_value = 0
+        max_retries = replica_settings.get('CQRS_MAX_RETRIES', DEFAULT_CQRS_MAX_RETRIES)
+        if not isinstance(max_retries, int) or max_retries < min_value:
+            logger.warning(
+                "Replica setting CQRS_MAX_RETRIES=%s is invalid, using default %s",
+                max_retries, DEFAULT_CQRS_MAX_RETRIES,
+            )
+            max_retries = DEFAULT_CQRS_MAX_RETRIES
+
+        return current_retry < max_retries
+
+    @staticmethod
+    def get_cqrs_retry_delay(current_retry):
+        """Returns number of seconds to wait before requeuing the message.
+
+        :param current_retry: Current number of message retries.
+        :type current_retry: int
+        :return: Delay in seconds.
+        :rtype: int
+        """
+        retry_delay = (
+            settings.CQRS
+            .get('replica', {})
+            .get('CQRS_RETRY_DELAY', DEFAULT_CQRS_RETRY_DELAY)
+        )
+
+        min_value = 1
+        if not isinstance(retry_delay, int) or retry_delay < min_value:
+            logger.warning(
+                "Replica setting CQRS_RETRY_DELAY=%s is invalid, using default %s",
+                retry_delay, DEFAULT_CQRS_RETRY_DELAY,
+            )
+            retry_delay = DEFAULT_CQRS_RETRY_DELAY
+
+        return retry_delay
